#ifndef UTILITIES_H
#define UTILITIES_H

#define _GNU_SOURCE
#include <inttypes.h>
#include <signal.h>
#include <stdint.h>
#include <time.h>
#include <unistd.h>

#ifdef WITH_XDP
#include <linux/if_ether.h>
#include <linux/if_link.h>
#include <linux/if_xdp.h>
#include <linux/ip.h>
#include <linux/udp.h>

#include <bpf/bpf.h>
#include <bpf/libbpf.h>
#include <bpf/xsk.h>
#endif

#define NSEC_PER_SEC UINT64_C(1000000000)
#define SERVER_PORT "50000"
#define SERVER_PORT_INT 50000
#define MAX_HIST_VAL 2000
#define NB_HISTOGRAMS 3
#define MAX_BUFFER_SIZE 1024

#define err(...)                  \
  do {                            \
    fprintf(stderr, __VA_ARGS__); \
    exit(EXIT_FAILURE);           \
  } while (0)

#define err_errno(...) error(EXIT_FAILURE, errno, __VA_ARGS__);

typedef struct packet_info {
  uint64_t userspace_enter_ts;
  uint64_t userspace_exit_ts;
  uint64_t kernelspace_ts;
  char data[MAX_BUFFER_SIZE];
} packet_info_t;

uint64_t ts_to_uint(struct timespec t);
void add_ns(struct timespec *t, uint64_t ns);
uint64_t calcdiff_ns(struct timespec t1, struct timespec t2);

void init_signals(void (*_sighand)(int), int enable_histograms);

extern void (*previous_handlers[NSIG])(int);

#endif
